#
# spec file for package valkey
#
# Copyright (c) 2025 SUSE LLC
# Copyright (c) Jonathan Wright
# Copyright (c) Neal Gompa
# Copyright (c) 2025 Andreas Stieger <Andreas.Stieger@gmx.de>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define _data_dir       %{_localstatedir}/lib/%{name}
%define _log_dir        %{_localstatedir}/log/%{name}
%define _conf_dir       %{_sysconfdir}/%{name}

%global make_flags CFLAGS="%{build_cflags}" DEBUG="" V="echo" PREFIX=%{buildroot}%{_prefix} BUILD_WITH_SYSTEMD=yes BUILD_TLS=yes

Name:           valkey
Version:        8.0.3
Release:        1.1
Summary:        Persistent key-value database
License:        BSD-3-Clause
URL:            https://valkey.io
Source0:        https://github.com/valkey-io/%{name}/archive/%{version}/%{name}-%{version}.tar.gz
Source1:        %{name}.logrotate
Source2:        %{name}.target
Source3:        %{name}@.service
Source4:        %{name}.tmpfiles.d
Source5:        README.SUSE
Source6:        %{name}.sysctl
Source7:        %{name}-sentinel@.service
Source8:        %{name}-sentinel.target
Source9:        %{name}-user.conf
Source10:       macros.%{name}
Source11:       migrate_redis_to_valkey.bash
# PATCH-FIX-OPENSUSE -- Adjust configs for openSUSE
Patch1001:      %{name}-conf.patch
BuildRequires:  jemalloc-devel
BuildRequires:  libopenssl-devel >= 1.1.1
BuildRequires:  pkgconfig
BuildRequires:  procps
BuildRequires:  python3
BuildRequires:  sysuser-shadow
BuildRequires:  sysuser-tools
BuildRequires:  tcl
BuildRequires:  pkgconfig(libsystemd)
BuildRequires:  pkgconfig(systemd)
# there is no tcl-tls package yet, which is said to be needed for testing tls support
Recommends:     logrotate
%{?sysusers_requires}

%global valkey_modules_abi 1
%global valkey_modules_dir %{_libdir}/%{name}/modules
Provides:       valkey(modules_abi)%{?_isa} = %{valkey_modules_abi}

%description
Valkey is an advanced key-value store. It is similar to memcached but the dataset
is not volatile, and values can be strings, exactly like in memcached,
but also lists, sets, and ordered sets. All this data types can be manipulated
with atomic operations to push/pop elements, add/remove elements, perform server
side union, intersection, difference between sets, and so forth. It supports many
different kinds of sorting abilities.

%package        devel
Summary:        Development header for Valkey module development

%description    devel
Header file required for building loadable Valkey modules.

%package        compat-redis
Summary:        Conversion script and compatibility symlinks for Redis
Requires:       valkey = %{version}-%{release}
Conflicts:      redis
Obsoletes:      redis <= 7.2.5
Provides:       redis = %{version}-%{release}
Requires(post): /usr/bin/find
BuildArch:      noarch

%description    compat-redis
This package contains compatibility symlinks and wrappers to enable
easy conversion from Redis to Valkey.

%prep
%autosetup -p1

# Module API version safety check
api=`sed -n -e 's/#define VALKEYMODULE_APIVER_[0-9][0-9]* //p' src/valkeymodule.h`
if test "$api" != "%{valkey_modules_abi}"; then
   : Error: Upstream API version is now ${api}, expecting %%{valkey_modules_abi}.
   : Update the valkey_modules_abi macro, the rpmmacros file, and rebuild.
   exit 1
fi

%build
%make_build %{make_flags}
%sysusers_generate_pre %{SOURCE9} %{name}

%install
%make_install %{make_flags}

install -pm0750 -d \
  %{buildroot}%{_log_dir} \
  %{buildroot}%{_data_dir} \
  %{buildroot}%{_conf_dir} \
  %{buildroot}%{_log_dir}/default \
  %{buildroot}%{_data_dir}/default

install -pDm644 src/%{name}module.h %{buildroot}%{_includedir}/%{name}module.h

install -Dpm0640 valkey.conf            %{buildroot}%{_conf_dir}/default.conf.example
install -Dpm0660 sentinel.conf          %{buildroot}%{_conf_dir}/sentinel.conf.example

# some sysctl stuff
install -Dpm0644 %{SOURCE6} %{buildroot}/%{_prefix}/lib/sysctl.d/00-%{name}.conf
%if 0%{?suse_version} > 1500
mkdir -p %{buildroot}%{_distconfdir}/logrotate.d
install -Dpm0644 %{SOURCE1} %{buildroot}%{_distconfdir}/logrotate.d/%{name}
%else
install -Dpm0644 %{SOURCE1} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}
%endif
install -Dpm0644 %{SOURCE2} %{buildroot}%{_unitdir}/%{name}.target
install -Dpm0644 %{SOURCE3} %{buildroot}%{_unitdir}/%{name}@.service
install -Dpm0644 %{SOURCE4} %{buildroot}%{_tmpfilesdir}/%{name}.conf
install -Dpm0644 %{SOURCE7} %{buildroot}%{_unitdir}/%{name}-sentinel@.service
install -Dpm0644 %{SOURCE8} %{buildroot}%{_unitdir}/%{name}-sentinel.target

cp %{SOURCE5} README.SUSE

mkdir -p %{buildroot}%{_sysusersdir}
install -pm0644 %{SOURCE9} %{buildroot}%{_sysusersdir}/

mkdir -p %{buildroot}%{_rpmmacrodir}
install -pm0644 %{SOURCE10} %{buildroot}%{_rpmmacrodir}/

mkdir -p %{buildroot}%{_libdir}/%{name}/modules

# Install valkey migration script
mkdir -p %{buildroot}%{_libexecdir}
install -pm0755 %{SOURCE11} %{buildroot}%{_libexecdir}/

# Install legacy compatibility paths in sbin
mkdir -p %{buildroot}%{_sbindir}
for redisbin in %{buildroot}%{_bindir}/redis-*; do
ln -sr $redisbin %{buildroot}%{_sbindir}/$(basename $redisbin)
done

%check
cat <<EOF
---------------------------------------------------
The test suite often fails to start a server, with
'child process exited abnormally' -- sometimes it works.
---------------------------------------------------
EOF
# Variable assignments need to match in all make invocations, otherwise it might recomplie. See https://github.com/redis/redis/issues/7337
%make_build test %{make_flags} || true

%pre -f %{name}.pre
%service_add_pre %{name}.target %{name}@.service %{name}-sentinel.target %{name}-sentinel@.service

%post
%tmpfiles_create %{_tmpfilesdir}/%{name}.conf
%service_add_post %{name}.target %{name}@.service %{name}-sentinel.target %{name}-sentinel@.service
echo "See %{_docdir}/%{name}/README.SUSE to continue"

%preun
%service_del_preun %{name}.target %{name}@.service %{name}-sentinel.target %{name}-sentinel@.service

%postun
%service_del_postun %{name}.target %{name}@.service %{name}-sentinel.target %{name}-sentinel@.service

%post compat-redis
%{_libexecdir}/migrate_redis_to_valkey.bash

%files
%license COPYING
%doc 00-RELEASENOTES README.md
%if 0%{?suse_version} > 1500
%{_distconfdir}/logrotate.d/%{name}
%else
%config(noreplace) %{_sysconfdir}/logrotate.d/%{name}
%endif
%{_prefix}/lib/sysctl.d/00-%{name}.conf
%{_bindir}/%{name}-*
%{_tmpfilesdir}/%{name}.conf
%{_sysusersdir}/%{name}-user.conf
%{_unitdir}/%{name}@.service
%{_unitdir}/%{name}.target
%{_unitdir}/%{name}-sentinel@.service
%{_unitdir}/%{name}-sentinel.target
%dir %{_libdir}/%{name}
%dir %{_libdir}/%{name}/modules
%doc README.SUSE
%config(noreplace) %attr(-,root,%{name}) %{_conf_dir}/
%dir %attr(0750,%{name},%{name}) %{_data_dir}
%dir %attr(0750,%{name},%{name}) %{_data_dir}/default
%dir %attr(0750,%{name},%{name}) %{_log_dir}
%ghost %dir %attr(0755,%{name},%{name}) /run/%{name}

%files devel
# main package is not required
%license COPYING
%{_includedir}/%{name}module.h
%{_rpmmacrodir}/macros.%{name}

%files compat-redis
%license COPYING
%{_libexecdir}/migrate_redis_to_valkey.bash
%{_bindir}/redis-*
%{_sbindir}/redis-*

%changelog
* Thu Apr 24 2025 Andreas Stieger <andreas.stieger@gmx.de>
- Update to 8.0.3:
  * CVE-2025-21605: Limit output buffer for unauthenticated clients
    (boo#1241708)
  * Optimize RDB load performance and fix cluster mode resizing on
    replica side
  * Fix memory leak in forgotten node ping ext code path
  * Fix cluster info sent stats for message with light header
  * Fix module LatencyAddSample still work when
    latency-monitor-threshold is 0
  * Fix potential crash in radix tree recompression of huge keys
  * Fix error "SSL routines::bad length" when connTLSWrite is called
    second time with smaller buffer
  * Fix temp file leak druing replication error handling
  * Fix ACL LOAD crash on replica since the primary client don't
    has a user
  * Fix RANDOMKEY infinite loop during CLIENT PAUSE
  * fix: add samples to stream object consumer trees
  * Fix cluster slot stats assertion during promotion of replica
  * Fix panic in primary when blocking shutdown after previous
    block with timeout
  * Ignore stale gossip packets that arrive out of order
  * Fix incorrect lag reported in XINFO GROUPS
  * Avoid shard id update of replica if not matching with primary
    shard id
* Wed Jan  8 2025 Antonio Teixeira <antonio.teixeira@suse.com>
- Update to 8.0.2:
  - Security fixes
  * (CVE-2024-46981, bsc#1235387) Lua script commands may lead to remote code execution. (#1513)
  * (CVE-2024-51741, bsc#1235386) Denial-of-service due to malformed ACL selectors. (#1514)
  - Bug fixes
  * Fix an uncommon crash when using TLS with dual channel replication. (#1173)
  * Make sure repl_down_since is correctly reset when dual channel replication
    fails. (#1149)
  * Fix a performance regression where a replica does not properly initialize
    the database size when loading a snapshot during replication. (#1199)
  * Make sure the last accessed time is correctly updated when using the TOUCH
    command with the CLIENT NO-TOUCH option.
  * Fix a bug where total_net_repl_output_bytes may report the wrong. (#1486)
  * Fix a bug where used_memory_scripts may report the wrong value. (#1255)
  * Fix a bug where server might crash when using active defrag when scripts
    are evicted from the script cache. (#1310)
  * Fix a bug where extra memory would be used when storing strings in the
    inline protocol. (#1213)
  * Fix a bug where the SORT command may throw a cross slot error. (#1182)
  * Fix a bug where the RANDOMKEY command may omit returning keys in cluster
    mode. (#1155)
  * Send the correct error message when FUNCTION KIlL is used to kill an
    ongoing script. (#1171)
  * Fix a potential memory corruption when databases are emptied, such as
    through FLUSHDB, when during active defrag is running. (#1512)
  - Behavior changes
  * Revert an unintended breaking change when sending an unsubuscribe command
    when a client is not subscribed to any channels. (#1265)
* Wed Dec  4 2024 Antonio Teixeira <antonio.teixeira@suse.com>
- Fix redis and sentinel units lookup in migration script (bsc#1226986)
  * Look for units in /etc/systemd/system/redis.target.wants
- Fix sentinel config files permissions in migration script
* Tue Oct 22 2024 Dirk Müller <dmueller@suse.com>
- fix requires for compat-redis package
* Fri Oct  4 2024 Antonio Teixeira <antonio.teixeira@suse.com>
- Update to 8.0.1:
  - Bug fixes
  * Fix a build issue with RDMA when using additional make parameters. (#1074)
  * Fix an issue where `CLUSTER SLOTS` might return the wrong tcp or tls port
    when called from inside a script or from a module. (#1072)
  * Fix a crash when `CLUSTER SLOTS` or `CLUSTER SHARDS` is called from inside
    a script or from a module. (#1063)
  * Fix a build issue on systems where `<threads.h>` is unavailable. (#1053)
  * Fix an issue with the default `sentinel.conf` being invalid. (#1040)
  - Security fixes
  * (CVE-2024-31449, bsc#1231264) Lua library commands may lead to stack
    overflow and potential RCE.
  * (CVE-2024-31227, bsc#1231266) Potential Denial-of-service due to malformed
    ACL selectors.
  * (CVE-2024-31228, bsc#1231265) Potential Denial-of-service due to unbounded
    pattern matching.
- 8.0.0 changelog:
  * See https://github.com/valkey-io/valkey/blob/8.0.0/00-RELEASENOTES
- Drop ppc-atomic.patch
- Refresh valkey-conf.patch
* Tue Aug 27 2024 Antonio Teixeira <antonio.teixeira@suse.com>
- Update to 7.2.6:
  - Bug Fixes - Core
  * Fix typo in REGISTER_API macro to prevent segfaults when loading Redis
    modules (#608)
  * Fix the command duration reset issue when clients are blocked and commands
    are reprocessed (#526)
  * Fix the data type conversion error in zrangeResultBeginStore (Redis#13148)
  * Fix a crash caused by quicklist node merges (Redis#13040)
  * Fix crashes in module blocking client timeout cases (Redis#13011)
  * Fix conversion of numbers in Lua args to Redis args
    (Redis#13115, Fixes Redis#13113)
  * Fix crash in LSET command when replacing small list items with larger ones,
    creating listpacks larger than 4GB (Redis#12955, Fixes Redis#12864)
  * Fix blocking command timeout reset issue during reprocessing (Redis#13004)
  - Bug Fixes - Cluster
  * Fix the CLUSTER SHARDS command to display accurate slot information even
    if a primary node fails (#790, Fixes #784)
  * Fix an issue where module authentication failed when the cluster was down
    (#693, Fixes #619)
  * Ensure only primary nodes with slots can mark another node as failed (#634)
  * Improve MEET command reliability under link failures to maintain cluster
    membership symmetry (#461)
  * Allow single primary node to mark potentially failed replica as FAIL in
    single-shard cluster (Redis#12824)
  - Bug Fixes - Sentinel
  * Accept redis-sentinel to start Valkey in sentinel mode (#731, Fixes #719)
  - Bug Fixes - CLI
  * Ensure the --count option in redis-cli works correctly even without
  - -pattern (Redis#13092)
  * Fix redis-check-aof misidentifying data in manifest format as MP-AOF
    (Redis#12951)
  * Update redis-check-rdb types to replace stream-v2 with stream-v3
    (Redis#12969)
* Tue Jul  9 2024 Antonio Teixeira <antonio.teixeira@suse.com>
- migrate_redis_to_valkey.bash fixes (boo#1227139)
  * Also set group owner for /var/lib/valkey
  * Change logdir option in copied conf files to /var/log/valkey
  * Change owner of copied conf files to root:valkey
* Tue Jul  2 2024 Andreas Schwab <schwab@suse.de>
- valkey.logrotate: fix owner of logfiles
* Sat Jun 29 2024 Neal Gompa <ngompa@opensuse.org>
- Add legacy symlinks for "redis" executables in sbin to fix
  running valkey in other package test suites using absolute paths
* Wed Jun 26 2024 Georg Pfuetzenreuter <georg.pfuetzenreuter@suse.com>
- Repair Redis instance detection (boo#1226986)
* Sat Jun 22 2024 Neal Gompa <ngompa@opensuse.org>
- Enable automatic replacement of redis with valkey
* Thu Jun  6 2024 Neal Gompa <ngompa@opensuse.org>
- Initial package based on the redis package and Fedora valkey package
- Ported over patch from redis package
  + Added valkey-conf.patch
- Backport patch from upstream submission
  + Added ppc-atomic.patch
